==> xdtp-1.1.2.tgz: xdtp-1.1.2/src/modules/XDTPModuleLoader.cpp <==
/* 
 * $Id: XDTPModuleLoader.cpp,v 1.8 2005/10/11 07:35:03 ozawa Exp $
 *
 * Copyright 2005- ONGS Inc. All rights reserved.
 * 
 * author: Masanori OZAWA (ozawa@ongs.co.jp)
 * version: $Revision: 1.8 $
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY ONGS INC ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ONGS INC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the ONGS Inc.
 * 
 */

#include "xdtp.h"
#include "modules/XDTPModuleLoader.h"

#include <dlfcn.h>

#define BUILDMODULE_SYM         "buildXDTPModule"
#define BUILDCHECKER_SYM        "buildXDTPChecker"
#define FREEMODULE_SYM          "freeXDTPModule"
#define FREECHECKER_SYM         "freeXDTPChecker"
#define GETMODULE_VERSION_SYM   "getXDTPModuleVersion"

using namespace XDTP;

XDTPModuleLoader::XDTPModuleLoader()
{
}

XDTPModuleLoader::~XDTPModuleLoader()
{
}

/**
 * XDTPModule񤹤饤֥ɤޤ
 * 
 * @param aPath XDTPModule񤹤ʥߥå󥯥饤֥ؤΥѥ
 * @return XDTPModule񤹤饤֥ꡣ
 *         פˤʤäʳǡunloadModuleLibraryɬפޤ
 */
XDTPModuleLibrary* XDTPModuleLoader::loadModuleLibrary(const Glib::ustring& aPath)
{
    void* handle = dlopen(aPath.c_str(), RTLD_NOW | RTLD_GLOBAL);
    
    if (!handle) {
        fprintf(stderr, "%s: %s\n", APP_NAME, dlerror());
        return NULL;
    }
    
    XDTPModuleLibrary* lib = new XDTPModuleLibrary;
    
    if (!lib) {
        dlclose(handle);
    }
    
    lib->handle = handle;
    lib->buildModule    = (XDTPModule*(*)())dlsym(handle, BUILDMODULE_SYM);
    lib->buildChecker   = (XDTPChecker*(*)())dlsym(handle, BUILDCHECKER_SYM);
    lib->freeModule     = (void(*)(XDTPModule*))dlsym(handle, FREEMODULE_SYM);
    lib->freeChecker    = (void(*)(XDTPChecker*))dlsym(handle, FREECHECKER_SYM);
    lib->getModuleVersion   = (u_int32_t(*)())dlsym(handle, GETMODULE_VERSION_SYM);
    
#ifdef DEBUG
    fprintf(stderr, "%s: handle: %p\n", APP_NAME, lib->handle);
    fprintf(stderr, "%s: buildModule: %p\n", APP_NAME, lib->buildModule);
    fprintf(stderr, "%s: buildChecker: %p\n", APP_NAME, lib->buildChecker);
    fprintf(stderr, "%s: freeModule: %p\n", APP_NAME, lib->freeModule);
    fprintf(stderr, "%s: freeChecker: %p\n", APP_NAME, lib->freeChecker);
    fprintf(stderr, "%s: getModuleVersion: %p\n", APP_NAME, lib->getModuleVersion);
#endif

    if (!lib->getModuleVersion ||
        (!lib->buildModule || !lib->freeModule) ||
        (lib->buildChecker && !lib->freeChecker))
    {
        dlclose(handle);
        delete lib;
        lib = NULL;
    }
    
    return lib;
}

/**
 * XDTPModule񤹤饤֥ޤ
 * ̤XDTPModule  XDTPChecker¸ߤϡޤ
 * 
 * @param aLibrary XDTPModule񤹤饤֥ꡣ
 */
void XDTPModuleLoader::unloadModuleLibrary(XDTPModuleLibrary* aLibrary)
{
    if (!aLibrary) {
        THROW("bad argument.");
    }
    
    {
        std::list<XDTPModule*>::iterator iter = (aLibrary->module).begin();
        std::list<XDTPModule*>::iterator end = (aLibrary->module).end();
        
        for (; iter != end; iter++) {
            aLibrary->freeModule(*iter);
        }
    }

    {
        std::list<XDTPChecker*>::iterator iter = (aLibrary->checker).begin();
        std::list<XDTPChecker*>::iterator end = (aLibrary->checker).end();
        
        for (; iter != end; iter++) {
            aLibrary->freeChecker(*iter);
        }
    }
    
    dlclose(aLibrary->handle);
    delete aLibrary;
}

/**
 * XDTPModuleۤޤ
 * 
 * @param aLibrary XDTPModule񤹤饤֥ꡣ
 * @return XDTPModuleᤷޤפˤʤäʳ freeModule ɬפޤ
 */
XDTPModule* XDTPModuleLoader::buildModule(XDTPModuleLibrary* aLibrary)
{
    if (!aLibrary) {
        THROW("bad argument.");
    }
    
    if (XDTP_MODULE_VERSION != aLibrary->getModuleVersion()) {
        return NULL;
    }
    
    XDTPModule* result = (XDTPModule*)aLibrary->buildModule();
    
    if (result) {
        (aLibrary->module).push_back(result);
    }
    
    return result;
}

/**
 * XDTPCheckerۤޤ
 * 
 * @param aLibrary XDTPChecker񤹤饤֥ꡣ
 * @return XDTPCheckerᤷޤפˤʤäʳ freeChecker ɬפޤ
 */
XDTPChecker* XDTPModuleLoader::buildChecker(XDTPModuleLibrary* aLibrary)
{
    if (!aLibrary) {
        THROW("bad argument.");
    }
    
    if (XDTP_MODULE_VERSION != aLibrary->getModuleVersion()) {
        return NULL;
    }

    if (!aLibrary->buildChecker) {
        return NULL;
    }

    XDTPChecker* result = (XDTPChecker*)aLibrary->buildChecker();
    
    if (result) {
        (aLibrary->checker).push_back(result);
    }
    
    return result;
}

/**
 * XDTPModuleޤ
 * 
 * @param aLibrary XDTPModule񤹤饤֥ꡣ
 * @param aModule ۤXDTPModule
 */
void XDTPModuleLoader::freeModule(XDTPModuleLibrary* aLibrary, XDTPModule* aModule)
{
    if (!aLibrary || !aModule) {
        THROW("bad argument.");
    }
    
    aLibrary->freeModule(aModule);
    (aLibrary->module).remove(aModule);
}

/**
 * XDTPCheckerޤ
 * 
 * @param aLibrary XDTPChecker񤹤饤֥ꡣ
 * @param aModule ۤXDTPChecker
 */
void XDTPModuleLoader::freeChecker(XDTPModuleLibrary* aLibrary, XDTPChecker* aChecker)
{
    if (!aLibrary || !aChecker) {
        THROW("bad argument.");
    }
    
    aLibrary->freeChecker(aChecker);
    (aLibrary->checker).remove(aChecker);
}

==> xdtp-1.1.4.tgz: xdtp-1.1.4/src/modules/XDTPModuleLoader.cpp <==
/* 
 * $Id: XDTPModuleLoader.cpp,v 1.9 2005/11/13 03:45:28 ozawa Exp $
 *
 * Copyright 2005- ONGS Inc. All rights reserved.
 * 
 * author: Masanori OZAWA (ozawa@ongs.co.jp)
 * version: $Revision: 1.9 $
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY ONGS INC ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ONGS INC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the ONGS Inc.
 * 
 */

#include "xdtp.h"
#include "modules/XDTPModuleLoader.h"

#include <dlfcn.h>

#define BUILDMODULE_SYM         "buildXDTPModule"
#define BUILDCHECKER_SYM        "buildXDTPChecker"
#define FREEMODULE_SYM          "freeXDTPModule"
#define FREECHECKER_SYM         "freeXDTPChecker"
#define GETMODULE_VERSION_SYM   "getXDTPModuleVersion"

using namespace XDTP;

XDTPModuleLoader::XDTPModuleLoader()
{
}

XDTPModuleLoader::~XDTPModuleLoader()
{
}

/**
 * XDTPModule񤹤饤֥ɤޤ
 * 
 * @param aPath XDTPModule񤹤ʥߥå󥯥饤֥ؤΥѥ
 * @return XDTPModule񤹤饤֥ꡣ
 *         פˤʤäʳǡunloadModuleLibraryɬפޤ
 */
XDTPModuleLibrary* XDTPModuleLoader::loadModuleLibrary(const Glib::ustring& aPath)
{
    void* handle = dlopen(aPath.c_str(), RTLD_NOW | RTLD_GLOBAL);
    
    if (!handle) {
        fprintf(stderr, "%s: %s\n", APP_NAME, dlerror());
        return NULL;
    }
    
    XDTPModuleLibrary* lib = new XDTPModuleLibrary;
    
    if (!lib) {
        dlclose(handle);
    }
    
    lib->handle = handle;
    lib->buildModule    = (XDTPModule*(*)())dlsym(handle, BUILDMODULE_SYM);
    lib->buildChecker   = (XDTPChecker*(*)())dlsym(handle, BUILDCHECKER_SYM);
    lib->freeModule     = (void(*)(XDTPModule*))dlsym(handle, FREEMODULE_SYM);
    lib->freeChecker    = (void(*)(XDTPChecker*))dlsym(handle, FREECHECKER_SYM);
    lib->getModuleVersion   = (u_int32_t(*)())dlsym(handle, GETMODULE_VERSION_SYM);
    
#ifdef DEBUG
    fprintf(stdout, "%s: module load result:\n", APP_NAME);
    fprintf(stdout, "\t handle: %p\n", lib->handle);
    fprintf(stdout, "\t buildModule: %p\n", lib->buildModule);
    fprintf(stdout, "\t buildChecker: %p\n", lib->buildChecker);
    fprintf(stdout, "\t freeModule: %p\n", lib->freeModule);
    fprintf(stdout, "\t freeChecker: %p\n", lib->freeChecker);
    fprintf(stdout, "\t getModuleVersion: %p\n", lib->getModuleVersion);
#endif

    if (!lib->getModuleVersion ||
        (!lib->buildModule || !lib->freeModule) ||
        (lib->buildChecker && !lib->freeChecker))
    {
        dlclose(handle);
        delete lib;
        lib = NULL;
    }
    
    return lib;
}

/**
 * XDTPModule񤹤饤֥ޤ
 * ̤XDTPModule  XDTPChecker¸ߤϡޤ
 * 
 * @param aLibrary XDTPModule񤹤饤֥ꡣ
 */
void XDTPModuleLoader::unloadModuleLibrary(XDTPModuleLibrary* aLibrary)
{
    if (!aLibrary) {
        THROW("bad argument.");
    }
    
    {
        std::list<XDTPModule*>::iterator iter = (aLibrary->module).begin();
        std::list<XDTPModule*>::iterator end = (aLibrary->module).end();
        
        for (; iter != end; iter++) {
            aLibrary->freeModule(*iter);
        }
    }

    {
        std::list<XDTPChecker*>::iterator iter = (aLibrary->checker).begin();
        std::list<XDTPChecker*>::iterator end = (aLibrary->checker).end();
        
        for (; iter != end; iter++) {
            aLibrary->freeChecker(*iter);
        }
    }
    
    dlclose(aLibrary->handle);
    delete aLibrary;
}

/**
 * XDTPModuleۤޤ
 * 
 * @param aLibrary XDTPModule񤹤饤֥ꡣ
 * @return XDTPModuleᤷޤפˤʤäʳ freeModule ɬפޤ
 */
XDTPModule* XDTPModuleLoader::buildModule(XDTPModuleLibrary* aLibrary)
{
    if (!aLibrary) {
        THROW("bad argument.");
    }
    
    if (XDTP_MODULE_VERSION != aLibrary->getModuleVersion()) {
        return NULL;
    }
    
    XDTPModule* result = (XDTPModule*)aLibrary->buildModule();
    
    if (result) {
        (aLibrary->module).push_back(result);
    }
    
    return result;
}

/**
 * XDTPCheckerۤޤ
 * 
 * @param aLibrary XDTPChecker񤹤饤֥ꡣ
 * @return XDTPCheckerᤷޤפˤʤäʳ freeChecker ɬפޤ
 */
XDTPChecker* XDTPModuleLoader::buildChecker(XDTPModuleLibrary* aLibrary)
{
    if (!aLibrary) {
        THROW("bad argument.");
    }
    
    if (XDTP_MODULE_VERSION != aLibrary->getModuleVersion()) {
        return NULL;
    }

    if (!aLibrary->buildChecker) {
        return NULL;
    }

    XDTPChecker* result = (XDTPChecker*)aLibrary->buildChecker();
    
    if (result) {
        (aLibrary->checker).push_back(result);
    }
    
    return result;
}

/**
 * XDTPModuleޤ
 * 
 * @param aLibrary XDTPModule񤹤饤֥ꡣ
 * @param aModule ۤXDTPModule
 */
void XDTPModuleLoader::freeModule(XDTPModuleLibrary* aLibrary, XDTPModule* aModule)
{
    if (!aLibrary || !aModule) {
        THROW("bad argument.");
    }
    
    aLibrary->freeModule(aModule);
    (aLibrary->module).remove(aModule);
}

/**
 * XDTPCheckerޤ
 * 
 * @param aLibrary XDTPChecker񤹤饤֥ꡣ
 * @param aModule ۤXDTPChecker
 */
void XDTPModuleLoader::freeChecker(XDTPModuleLibrary* aLibrary, XDTPChecker* aChecker)
{
    if (!aLibrary || !aChecker) {
        THROW("bad argument.");
    }
    
    aLibrary->freeChecker(aChecker);
    (aLibrary->checker).remove(aChecker);
}
